#!/bin/perl

=head1 NAME

 RabbitMQ_Nodes.pl 
 
=head1 SYNOPSIS

 IntroscopeEPAgent.properties configuration
 
 introscope.epagent.stateless.names=NODES
 introscope.epagent.stateless.NODES.command=perl <epa_home>/epaplugins/rabbitmq/RabbitMQ_Nodes.pl --host host_or_ip_addr --port 12345
 introscope.epagent.stateless.NODES.delayInSeconds=15

=head1 DESCRIPTION

 Pulls statistics about bindings
 
 To see help information:
 
 perl <epa_home>/epaplugins/rabbitmq/RabbitMQ_Nodes.pl --help
 
 or run with no commandline arguments.
 
 To test against sample output, use the DEBUG flag:
 
 perl <epa_home>/epaplugins/rabbitmq/RabbitMQ_Nodes.pl --debug

=head1 CAVEATS

 For Windows users, please use foward slash "/" as a path separator.
 If directory names have spaces, use the 8.3 format/shortname instead.
 Type "dir /x" from a command prompt to see the shortnames.
 
 IMPORTANT!! Place a copy of rabbitmqadmin.py in the same location as this plug-in.

=head1 AUTHOR
 
 Hiko Davis, Principal Services Consultant, CA Technologies

=head1 COPYRIGHT

 Copyright (c) 2014
 
 This plug-in is provided AS-IS, with no warranties, so please test thoroughly!
   
=cut

use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/lib/perl", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;

use strict;
use warnings;

use Getopt::Long;
use File::Spec;
use Cwd qw(abs_path);


sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "usage: $0 [--host HOST] [--port PORT] [--help|-?]\n";
    exit;
}

my ($help, $rmqHost, $rmqPort, $debug);

# get commandline parameters or display help
usage () if ( @ARGV < 1 or 
    ! GetOptions( 'help|?'  =>  \$help,
                  'host=s'  =>  \$rmqHost,
                  'port=i'  =>  \$rmqPort,
                  'debug!'  =>  \$debug,
                )
    or defined $help );

my (@arrayResults, $execCommand, $rabbitmqadmin);

if ($debug) {
    # read in the test output file; adjust path as needed for your environment
    @arrayResults = do { open my $fh, '<', File::Spec->catfile(abs_path, "RabbitMQ", "samples", "RabbitMQ_Nodes"); <$fh>; };
} else {
    # determine path to rabbitmqadmin.py; adjust path as needed for your environment
    $rabbitmqadmin = File::Spec->catfile(abs_path, "RabbitMQ", "rabbitmqadmin.py");
    # command to execute rabbitmqadmin.py
    $execCommand="python $rabbitmqadmin --host=$rmqHost --port=$rmqPort --format=nodes list nodes";
    # execute command, place results into array
    @arrayResults=`$execCommand`;
}

# instantiate counter for loop
my $i = 0;

foreach my $line ( @arrayResults ) {
    # removing trailing newline
    chomp $line;
    # split line on colon ":"
    (undef, my $Data) = split(':', $line, 2);
    # split Data on equals "="
    my ($metricName, $metricValue) = split('=', $Data);
    # return results
    if ( $i <= 1 || $i == 4 || $i >= 9 && $i <= 10 || $i == 13 || $i == 18 || $i >= 23 && $i <= 24 ) {
        # return 'Uknown' if no value was assigned file splitting Data
        $metricValue = 'Unknown' if ( length($metricValue) == 0 );
        # result value is string
        Wily::PrintMetric::printMetric(  type        => 'StringEvent',
                                         resource    => 'RabbitMQ',
                                         subresource => 'Nodes',
                                         name        => $metricName,
                                         value       => $metricValue,
                                      );
    } elsif ( $i == 16 || $i >= 20 && $i <= 22 ) {
        #TODO parse the named pairs
        # get rid of square brackets
        # split metricValue on close curly brace+comma+space "}, "
    } else {
        # return 0 if no value was assigned file splitting Data
        $metricValue = 0 if ( length($metricValue) == 0 );
        # result value is long
        Wily::PrintMetric::printMetric(  type        => 'LongCounter',
                                         resource    => 'RabbitMQ',
                                         subresource => 'Nodes',
                                         name        => $metricName,
                                         value       => $metricValue,
                                      );
    }
    # increment counter
    $i++;
}
